# Utility functions

This module contains functions that extend the functionality of standard data types, other types of utility functionality get their own module

```idris
module Util

import Data.SortedSet

%default total
```

## Either

<!-- idris
namespace Either
-->

### mapLeft

Applies a function to the contents of a `Left` if the value of the given `Either` is actually a `Left`, otherwise, leaves `Right`s untouched.

```idris
  export
  mapLeft : (f : a -> b) -> Either a c -> Either b c
  mapLeft f (Left x) = Left (f x)
  mapLeft f (Right x) = Right x
```
## Vectors

Define some operations for pairs of numbers, treating them roughly like vectors

### Addition and Subtraction

<!-- idris
export infixl 8 >+<
export infixl 8 >-<

namespace Pair
-->

```idris
  public export
  (>+<) : Num a => (x, y : (a, a)) -> (a, a)
  (x_1, x_2) >+< (y_1, y_2) = (x_1 + y_1, x_2 + y_2)

  public export
  (>-<) : Neg a => (x, y : (a, a)) -> (a, a)
  (x_1, x_2) >-< (y_1, y_2) = (x_1 - y_1, x_2 - y_2)
```

## Set

Extend `Data.SortedSet`

<!-- idris
namespace Set
-->

### length

Count the number of elements in a sorted set

```idris
  export
  length : SortedSet k -> Nat
  length x = foldl (\acc, e => acc + 1) 0 x
```
